# -*- coding: utf-8 -*-
import consulta
import pandas as pd
import numpy as np
import time
from datetime import date
import dateutil.relativedelta

###########################################################################################################################
### monta dataframe para os cardapios
###########################################################################################################################
def geraPlanilha():
    print('processo iniciado')
    total_time = time.time()
    sqlCardapios = consulta.cardapioConsulta()
    
    print('Montando dataframe cardapio')
    cardapio_trans_time = time.time()
    
    mesAtual = date.today().replace(day=1)
    mes_12 = pd.to_datetime(mesAtual.replace(year=mesAtual.year-1), infer_datetime_format=True)
    mes_6 = pd.to_datetime(mesAtual - dateutil.relativedelta.relativedelta(months=6), infer_datetime_format=True)
    mes_3 = pd.to_datetime(mesAtual - dateutil.relativedelta.relativedelta(months=3), infer_datetime_format=True)
    
    #montando dataframe para cardapios
    dfCardapios = pd.DataFrame(sqlCardapios, columns=['FILISERV', 'CDFILAUXPRAT', 'CDPRATO', 'DTMESCARD'])   
    
    #cria colunas para quantidade de ocorrencias nos ultimos 12, 6 e 3 meses, coloca valores 1 ou 0 para depois somar
    dfCardapios['QTD_UTIL_12'] = np.where((dfCardapios['DTMESCARD'] >= mes_12) & (dfCardapios['DTMESCARD'] < mes_6) ,1, 0)
    dfCardapios['QTD_UTIL_6'] = np.where((dfCardapios['DTMESCARD'] >= mes_6) & (dfCardapios['DTMESCARD'] < mes_3) ,1, 0)
    dfCardapios['QTD_UTIL_3'] = np.where(dfCardapios['DTMESCARD'] >= mes_3 ,1, 0)
    
    #cria as colunas para os ultimos 7 meses, 1 por mes
    qtd_1 = 'QTD_' + str((mesAtual - dateutil.relativedelta.relativedelta(months=1)).month)+'_'+str((mesAtual - dateutil.relativedelta.relativedelta(months=1)).year)
    qtd_2 = 'QTD_' + str((mesAtual - dateutil.relativedelta.relativedelta(months=2)).month)+'_'+str((mesAtual - dateutil.relativedelta.relativedelta(months=2)).year)
    qtd_3 = 'QTD_' + str((mesAtual - dateutil.relativedelta.relativedelta(months=3)).month)+'_'+str((mesAtual - dateutil.relativedelta.relativedelta(months=3)).year)
    qtd_4 = 'QTD_' + str((mesAtual - dateutil.relativedelta.relativedelta(months=4)).month)+'_'+str((mesAtual - dateutil.relativedelta.relativedelta(months=4)).year)
    qtd_5 = 'QTD_' + str((mesAtual - dateutil.relativedelta.relativedelta(months=5)).month)+'_'+str((mesAtual - dateutil.relativedelta.relativedelta(months=5)).year)
    qtd_6 = 'QTD_' + str((mesAtual - dateutil.relativedelta.relativedelta(months=6)).month)+'_'+str((mesAtual - dateutil.relativedelta.relativedelta(months=6)).year)
    qtd_7 = 'QTD_' + str((mesAtual - dateutil.relativedelta.relativedelta(months=7)).month)+'_'+str((mesAtual - dateutil.relativedelta.relativedelta(months=7)).year)
    
    dfCardapios[qtd_1] = np.where(dfCardapios['DTMESCARD'] == pd.to_datetime(mesAtual - dateutil.relativedelta.relativedelta(months=1), infer_datetime_format=True) ,1, 0)
    dfCardapios[qtd_2] = np.where(dfCardapios['DTMESCARD'] == pd.to_datetime(mesAtual - dateutil.relativedelta.relativedelta(months=2), infer_datetime_format=True) ,1, 0)
    dfCardapios[qtd_3] = np.where(dfCardapios['DTMESCARD'] == pd.to_datetime(mesAtual - dateutil.relativedelta.relativedelta(months=3), infer_datetime_format=True) ,1, 0)
    dfCardapios[qtd_4] = np.where(dfCardapios['DTMESCARD'] == pd.to_datetime(mesAtual - dateutil.relativedelta.relativedelta(months=4), infer_datetime_format=True) ,1, 0)
    dfCardapios[qtd_5] = np.where(dfCardapios['DTMESCARD'] == pd.to_datetime(mesAtual - dateutil.relativedelta.relativedelta(months=5), infer_datetime_format=True) ,1, 0)
    dfCardapios[qtd_6] = np.where(dfCardapios['DTMESCARD'] == pd.to_datetime(mesAtual - dateutil.relativedelta.relativedelta(months=6), infer_datetime_format=True) ,1, 0)
    dfCardapios[qtd_7] = np.where(dfCardapios['DTMESCARD'] == pd.to_datetime(mesAtual - dateutil.relativedelta.relativedelta(months=7), infer_datetime_format=True) ,1, 0)
    
    #agrupando as quantidades por 12m 6m e 3m e os ultimos 7m um por mes, por prato e filial
    cardapios = dfCardapios[['FILISERV', 'CDFILAUXPRAT', 'CDPRATO','QTD_UTIL_12','QTD_UTIL_6','QTD_UTIL_3',qtd_7,qtd_6,qtd_5,qtd_4,qtd_3,qtd_2,qtd_1]].groupby(['FILISERV', 'CDFILAUXPRAT', 'CDPRATO']).sum().reset_index()
    
    cardapio_trans_time = time.time() - cardapio_trans_time
    print('Dataframe cardapio montado em:',consulta.tempo(cardapio_trans_time))
    
    ###########################################################################################################################
    ### monta dataframe para as receitas
    ###########################################################################################################################
    sqlReceitas = consulta.receitaConsulta();
    
    print('Montando dataframe receitas')
    receita_trans_time = time.time()
    #criacao do dataframe para as receitas ativas
    receitas = pd.DataFrame(sqlReceitas, columns=['FILISERV', 'CDFILAUXPRAT', 'CDPRATO', 'NMPRATO', 'RECEITA'])   
    
    #mescla o dataframe de cardapios com o de receitas
    result = pd.merge(receitas, cardapios, on=['FILISERV', 'CDFILAUXPRAT', 'CDPRATO'], how='left').fillna(0)
    
    #agrupando as quantidades por 12m 6m e 3m e os ultimos 7m um por mes, por receita
    result = result[['CDFILAUXPRAT', 'CDPRATO','NMPRATO', 'RECEITA','QTD_UTIL_12','QTD_UTIL_6','QTD_UTIL_3',qtd_7,qtd_6,qtd_5,qtd_4,qtd_3,qtd_2,qtd_1]].groupby(['CDFILAUXPRAT', 'CDPRATO', 'NMPRATO', 'RECEITA']).sum().reset_index()
    
    #concatena filiais/servicos na coluna filiserv (agrupa dados por receita e prato)
    receitas = receitas.groupby(['CDFILAUXPRAT', 'CDPRATO', 'NMPRATO', 'RECEITA'],as_index=False)[['FILISERV']].aggregate(lambda x: set(x))
    
    #mescla novamente o dataframe de receitas com o dataframe result que tem as quantidades de ocorrencias de cardápio ja agrupadas
    receitas = pd.merge(receitas, result, on=['CDFILAUXPRAT', 'CDPRATO', 'NMPRATO', 'RECEITA'], how='left').fillna(0)
    
    #acrescenta coluna de quantidade de filiais/servicos que utilizam cada receita
    receitas['COUNT'] = receitas.FILISERV.apply(lambda x: len(x))
    
    #cria a coluna de quantidade de repetição de cada receita
    receitas['concat'] = receitas['NMPRATO'] + receitas['RECEITA']
    
    #cria a coluna de quantidade de repetição de cada receita
    receitas['QTDE_REP'] = receitas.groupby(['NMPRATO','RECEITA'])['concat'].transform(len)
    
    #exclui a coluna 'concat' desnecessaria
    receitas = receitas.drop(columns=['concat'])
    
    #ordena o dataset para criar o conj de repeticao
    receitas = receitas.sort_values(by=['QTDE_REP','NMPRATO'],ascending=[False,True]).reset_index()
    
    #cria a coluna conjunto repeticao, que cria um indice para cada receita que se repete
    receitas['CONJ_REPETICAO'] = receitas.groupby(['NMPRATO','RECEITA']).ngroup(ascending=True)
    
    #cria a coluna para a quantidade de repetição de cada conjunto de repetição
    receitas['QTDE_REPETICAO'] = receitas.groupby(['CONJ_REPETICAO'])['CONJ_REPETICAO'].transform('size')
    
    #limpa o campo de conjunto de repeticao se só existir 1 repetição da receita e nome do prato
    receitas['CONJ_REPETICAO'] = np.where(receitas['QTDE_REP']==1, '', receitas['CONJ_REPETICAO'])
    
    #cria uma coluna temp para retornar a quantidade de pratos por nome
    receitas['QT'] = receitas.groupby(['NMPRATO'])['NMPRATO'].transform('size')
      
    #cria a coluna de posicao do prato ordenado pela quantidade de pratos por nome
    receitas['POS'] = receitas.groupby(['QT','NMPRATO']).ngroup(ascending=False)
    
    #seleciona novamente as colunas para o dataframe de receitas
    receitas = receitas[['POS','NMPRATO','CDFILAUXPRAT','CDPRATO','QTDE_REPETICAO','CONJ_REPETICAO','RECEITA','COUNT','FILISERV', 'QTD_UTIL_12','QTD_UTIL_6','QTD_UTIL_3',qtd_7,qtd_6,qtd_5,qtd_4,qtd_3,qtd_2,qtd_1]].sort_values(by=['POS','COUNT'],ascending=[True,False])
    
    receita_trans_time = time.time() - receita_trans_time
    print('Dataframe receitas montado em:',consulta.tempo(receita_trans_time))
    
    ###########################################################################################################################
    ### escrevendo a planilha final (na pasta export criada a partir da raiz)
    ###########################################################################################################################
    print('criando arquivo xlsx "base_receitas_ativas_teknisa"')
    xls_writer_time = time.time()
    # Create a Pandas Excel writer using XlsxWriter as the engine.
    writer = pd.ExcelWriter('./export/base_receitas_ativas_teknisa.xlsx', engine='xlsxwriter')
    now = date.today()
    stSheetName = 'Gerado em '+ now.strftime('%d-%m-%Y')
    # Convert the dataframe to an XlsxWriter Excel object.
    receitas.to_excel(writer, sheet_name=stSheetName, index=False)
    
    workbook  = writer.book
    worksheet = writer.sheets[stSheetName]
    
    header_fmt = workbook.add_format({'bg_color': 'caf0f8', 'bold': True, 'border': True})
    worksheet.set_row(0, None, header_fmt)    

    for columnnum, columnname in enumerate(list(receitas.columns)):
        worksheet.write(0, columnnum, columnname, header_fmt)
    # Close the Pandas Excel writer and output the Excel file.

    writer.save()
    xls_writer_time = time.time() - xls_writer_time
    print('Arquivo criado em:',consulta.tempo(xls_writer_time))
    total_time = time.time() - total_time
    print('processo concluido. Tempo total:',consulta.tempo(total_time) )

geraPlanilha()
